/*
Bullet Continuous Collision Detection and Physics Library
Copyright (c) 2003-2006 Erwin Coumans  http://continuousphysics.com/Bullet/

This software is provided 'as-is', without any express or implied warranty.
In no event will the authors be held liable for any damages arising from the use of this software.
Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it freely,
subject to the following restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
3. This notice may not be removed or altered from any source distribution.
*/
/*
MODIFICATION of GlutStuff.cpp.
// IMO gDemoApplication->initPhysics() MUST be called AFTER a gl rendering context is present and myinit has been called. To do that a
// re-implementation of the whole "GlutStuff.cpp" is necessary since static global methods can't be declared extern:
USAGE:
Just use glutmainEx instead of glutmain.
*/
#ifndef _WINDOWS


#include "GlutDemoApplicationEx.h"
#include "GLDebugDrawer.h"
#include <BulletDynamics/Dynamics/btDynamicsWorld.h>
#ifdef BT_USE_FREEGLUT
#include <GL/freeglut_ext.h>
#endif //BT_USE_FREEGLUT

static GlutDemoApplicationEx* gDemoApplication;
static	void glutKeyboardCallback(unsigned char key, int x, int y){
	gDemoApplication->keyboardCallback(key,x,y);
}
static	void glutKeyboardUpCallback(unsigned char key, int x, int y){
  gDemoApplication->keyboardUpCallback(key,x,y);
}
static void glutSpecialKeyboardCallback(int key, int x, int y){
	gDemoApplication->specialKeyboard(key,x,y);
}
static void glutSpecialKeyboardUpCallback(int key, int x, int y){
	gDemoApplication->specialKeyboardUp(key,x,y);
}
static void glutReshapeCallback(int w, int h){
	gDemoApplication->reshape(w,h);
}
static void glutMoveAndDisplayCallback(){
	gDemoApplication->moveAndDisplay();
}
static void glutMouseFuncCallback(int button, int state, int x, int y){
//#ifdef __FREEGLUT_EXT_H__
    #if (!defined(_WIN32) && !defined(_WIN64))
    if (button == 3 || button == 4) {
        //if (state==GLUT_DOWN)
        gDemoApplication->mouseWheelFunc(button == 3 ? 1 : -1,button == 3 ? 1 : -1,x,y);
        //gDemoApplication->mouseWheelFunc(state==GLUT_DOWN ? 1 : -1,state,x,y);
        return;
    }
    #endif
//#endif
    gDemoApplication->mouseFunc(button,state,x,y);
}
static void	glutMotionFuncCallback(int x,int y){
    gDemoApplication->mouseMotionFunc(x,y);
}
//#ifdef __FREEGLUT_EXT_H__
static void glutMouseWheelFuncCallback(int a,int b,int c,int d)	{
    #if (defined(_WIN32) || defined(_WIN64))
    gDemoApplication->mouseWheelFunc(a,b,c,d);
    #endif
}
//#endif //__FREEGLUT_EXT_H__
static void glutDisplayCallback(void){
	gDemoApplication->displayCallback();
}

int glutmainEx(int argc, char **argv,int width,int height,const char* title,GlutDemoApplicationEx* demoApp,GLDebugDrawer* debugDrawer=NULL) {

	gDemoApplication = demoApp;

	glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH | GLUT_STENCIL);
    glutInitWindowPosition(0, 0);
    glutInitWindowSize(width, height);

    glutCreateWindow(title);
#ifdef BT_USE_FREEGLUT
	glutSetOption (GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_GLUTMAINLOOP_RETURNS);
#endif

    gDemoApplication->myinit();
	gDemoApplication->initPhysics();
	if (gDemoApplication->getDynamicsWorld()) gDemoApplication->getDynamicsWorld()->setDebugDrawer(debugDrawer);

	glutKeyboardFunc(glutKeyboardCallback);
	glutKeyboardUpFunc(glutKeyboardUpCallback);
	glutSpecialFunc(glutSpecialKeyboardCallback);
	glutSpecialUpFunc(glutSpecialKeyboardUpCallback);

	glutReshapeFunc(glutReshapeCallback);
    //createMenu();
	glutIdleFunc(glutMoveAndDisplayCallback);
	glutMouseFunc(glutMouseFuncCallback);
	glutPassiveMotionFunc(glutMotionFuncCallback);
	glutMotionFunc(glutMotionFuncCallback);
	glutDisplayFunc( glutDisplayCallback );

	#ifdef __FREEGLUT_EXT_H__
	glutMouseWheelFunc(glutMouseWheelFuncCallback);
	#endif //__FREEGLUT_EXT_H__

	glutMoveAndDisplayCallback();

//enable vsync to avoid tearing on Apple (todo: for Windows)

#if defined(__APPLE__) && !defined (VMDMESA)
int swap_interval = 1;
CGLContextObj cgl_context = CGLGetCurrentContext();
CGLSetParameter(cgl_context, kCGLCPSwapInterval, &swap_interval);
#endif


	glutMainLoop();

    gDemoApplication->exitPhysics();
    return 0;
}


#endif //_WINDOWS
